class SmartDMConfigWindow extends UWindowPageWindow;

#exec texture IMPORT NAME=bgnd File=Textures\bgnd.pcx GROUP=SmartSB MIPS=OFF

var UWindowSmallButton ButClose;
var UWindowCheckbox ShowTransparent;
var UWindowCheckbox ShowAltBoard;
var UWindowCheckbox ShowStatFaces;
var UWindowCheckbox ShowLineBoard;
var UWindowCheckbox ShowAltHeading;
var UWindowCheckbox ShowCrossHair;
var UWindowCheckbox ShowFrameRate;

var color TxtColor, GrayColor;

var SmartDMGameReplicationInfo SDMGame;

function Created()
{
     Super.Created();

     if ( SDMGame == none )
     {
        foreach Root.GetPlayerOwner().AllActors(class'SmartDMGameReplicationInfo', SDMGame)
        break;
     }

     ShowTransparent = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,35.0,215.0, 1));
     ShowTransparent.TextColor = TxtColor;
     ShowTransparent.SetText("Show Transparent Scoreboard.........");
     ShowTransparent.bChecked = class'SmartDMScoreBoard'.default.bShowTransparent;

     ShowAltBoard = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,55.0,215.0, 1));
     ShowAltBoard.TextColor = TxtColor;
     ShowAltBoard.SetText(   "Show Alternate Scoreboard..............");
     ShowAltBoard.bChecked = class'SmartDMScoreBoard'.default.bShowAltScoreboard;

     ShowStatFaces = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,75.0,215.0, 1));
     ShowStatFaces.TextColor = TxtColor;
     ShowStatFaces.SetText(  "Show Scoreboard Faces...................");
     ShowStatFaces.bChecked = class'SmartDMScoreBoard'.default.bStatsDrawFaces;

     ShowLineBoard = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,95.0,215.0, 1));
     if (SDMGame.bTeamGame)
        ShowLineBoard.TextColor = GrayColor;
     else
        ShowLineBoard.TextColor = TxtColor;
     ShowLineBoard.SetText(  "Show DM Players Inline....................");
     ShowLineBoard.bChecked = class'SmartDMScoreBoard'.default.bShowInlineDMPlayers;
     ShowLineBoard.bDisabled = SDMGame.bTeamGame;

     ShowAltHeading = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,115.0,215.0, 1));
     if ( SDMGame.bTeamGame  && !SDMGame.bCTFGame)
        ShowAltHeading.TextColor = TxtColor;
     else
        ShowAltHeading.TextColor = GrayColor;
     ShowAltHeading.SetText( "Show Alternate TDM Headings..........");
     ShowAltHeading.bChecked = class'SmartDMScoreBoard'.default.bShowAltTeamHeading;
     if ( !SDMGame.bTeamGame || SDMGame.bCTFGame )
        ShowAltHeading.bDisabled = True;

     ShowCrossHair = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,135.0,215.0, 1));
     ShowCrossHair.TextColor = TxtColor;
     ShowCrossHair.SetText(  "Show Crosshair in Scoreboard..........");
     ShowCrossHair.bChecked = class'SmartDMScoreBoard'.default.bShowSBCrossHair;

     ShowFrameRate = UWindowCheckbox(CreateControl(class'UWindowCheckbox', 40.0,155.0,215.0, 1));
     ShowFrameRate.TextColor = TxtColor;
     ShowFrameRate.SetText(  "Show Tick and Frame Rates...............");
     ShowFrameRate.bChecked = class'SmartDMScoreBoard'.default.bShowTickFrameRate;

     ButClose = UWindowSmallButton(CreateControl(class'UWindowSmallButton', 110.0,185.0,80.0,35.0));
     ButClose.SetText("Close");
     ButClose.DownSound = sound'UnrealShare.FSHLITE2';
}

function WriteText(canvas C, string text, int q)
{
    local float W, H;
    TextSize(C, text, W, H);
    ClipText(C, 10, q, text, false);
}

function Paint(Canvas C, float X, float Y)
{
    Super.Paint(C,X,Y);

     C.DrawColor.R = 0;
     C.DrawColor.G = 0;
     C.DrawColor.B = 128;

     DrawStretchedTexture(C, 0, 0, WinWidth, WinHeight, Texture'bgnd');

     C.DrawColor.R = 255;
     C.DrawColor.G = 255;
     C.DrawColor.B = 255;

     C.Font = Font( DynamicLoadObject( "LadderFonts.UTLadder18", class'Font' ) );
     ClipText(C, 60, 5,  "SmartDM Scoreboard");
}

function Notify(UWindowDialogControl C, byte E)
{
     local String str;

     Switch(E)
     {
	   case DE_Click:
		 switch(C)
         {
		  case ButClose:
               ParentWindow.ParentWindow.Close();
               break;
         }
          break;

       case DE_Change:    // Now the settings are saved on just ticking or unticking
         switch(C)
         {
          case ShowTransparent:
               saveSettings();
               break;
          case ShowAltBoard:
               saveSettings();
               break;
          case ShowStatFaces:
               saveSettings();
               break;
          case ShowLineBoard:
               saveSettings();
               break;
          case ShowAltHeading:
               saveSettings();
               break;
          case ShowCrossHair:
               saveSettings();
               break;
          case ShowFrameRate:
               saveSettings();
               break;
         }
          break;
		// these are used to display tooltips
	   case DE_MouseMove:
			if ( UMenuRootWindow(Root) != None && UMenuRootWindow(Root).StatusBar != None)
			   UMenuRootWindow(Root).StatusBar.SetHelp(C.HelpText);
			break;
	   case DE_MouseLeave:
			if ( UMenuRootWindow(Root) != None && UMenuRootWindow(Root).StatusBar != None)
			   UMenuRootWindow(Root).StatusBar.SetHelp("");
			break;
     }

     Super.Notify(C,E);
}

function saveSettings()
{
      local SmartDMScoreBoard SB;

      class'SmartDMScoreBoard'.default.bShowTransparent = ShowTransparent.bChecked;
      class'SmartDMScoreBoard'.default.bShowAltScoreboard = ShowAltBoard.bChecked;
      class'SmartDMScoreBoard'.default.bStatsDrawFaces = ShowStatFaces.bChecked;
      class'SmartDMScoreBoard'.default.bShowInlineDMPlayers = ShowLineBoard.bChecked;
      class'SmartDMScoreBoard'.default.bShowAltTeamHeading = ShowAltHeading.bChecked;
      class'SmartDMScoreBoard'.default.bShowSBCrossHair = ShowCrossHair.bChecked;
      class'SmartDMScoreBoard'.default.bShowTickFrameRate = ShowFrameRate.bChecked;
      class'SmartDMScoreBoard'.static.StaticSaveConfig();  //Settings saved, but what about current scoreboard? :P

      foreach GetPlayerOwner().AllActors(class'SmartDMScoreBoard', SB)  // Goccha!!
        break;

      if (SB == none)
         return;

      SB.bShowTransparent =  ShowTransparent.bChecked;
      SB.bShowAltScoreboard = ShowAltBoard.bChecked;
      SB.bStatsDrawFaces = ShowStatFaces.bChecked;
      SB.bShowInlineDMPlayers = ShowLineBoard.bChecked;
      SB.bShowAltTeamHeading = ShowAltHeading.bChecked;
      SB.bShowSBCrossHair = ShowCrossHair.bChecked;
      SB.bShowTickFrameRate = ShowFrameRate.bChecked;
}

function Close(optional bool bByParent)
{
    local int w, Mode;
    Super.Close(bByParent);
}

defaultproperties
{
     TxtColor=(R=255,G=255,B=255)
     GrayColor=(R=64,G=64,B=64)
}
