class SmartDMMenuInfo extends Info;

var() class <UWindowWindow> WindowClass;

var() int WinLeft, WinTop, WinWidth, WinHeight;

var() bool DestroyOnClose;

var UWindowWindow TheWindow;

var string Version;

var int TicksPassed;

event PostBeginPlay()
{
	Super.PostBeginPlay();
	OpenIfNecessary();
}

simulated event PostNetBeginPlay()
{
	Super.PostBeginPlay();
	OpenIfNecessary();
}

simulated function OpenIfNecessary()
{
    local PlayerPawn P;

    if ( Owner != None )
    {
         P = PlayerPawn(Owner);
         if ( P!=None && P.Player!=None && P.Player.Console !=None )
         {
             OpenWindow();
         }
    }
}

simulated function bool OpenWindow()
{
    local PlayerPawn P;
    local WindowConsole C;

    P = PlayerPawn(Owner);
    if (P == None)
    {
         log("SmartDMMenuInfo Attempted to open a window on something other than a PlayerPawn", 'SmartDM');
         DestroyWRI();
         return false;
    }

    C = WindowConsole(P.Player.Console);
    if (C == None)
    {
         Log("SmartDMMenuInfo - No Console", 'SmartDM');
         DestroyWRI();
         return false;
    }

    if (!C.bCreatedRoot || C.Root == None)
    {
         C.CreateRootWindow(None);
    }

    C.bQuickKeyEnable = true;
    C.LaunchUWindow();
    TicksPassed = 1;
    return true;
}

simulated function Tick(float DeltaTime)
{
    if (TicksPassed != 0)
    {
       if (TicksPassed++ == 2)
       {
          SetupWindow();
          TicksPassed = 0;            // Reset TicksPassed to 0
       }
    }
    if (DestroyOnClose && TheWindow != None && !TheWindow.bWindowVisible)
    {
       DestroyWRI();
    }
}

simulated function bool SetupWindow()
{
     local WindowConsole C;

     C = WindowConsole(PlayerPawn(Owner).Player.Console);

     TheWindow = C.Root.CreateWindow(WindowClass, WinLeft, WinTop, WinWidth, WinHeight);

     if (TheWindow == None)
     {
          DestroyWRI();  // create failed
          return false;
     }

     if ( C.bShowConsole )
          C.HideConsole();

     TheWindow.ShowWindow();
     TheWindow.bLeaveOnScreen = True;
     SetTimer(2, false);
     return true;
}

simulated function timer()
{
    SmartDMDialogWindow(TheWindow).SetupMenu(Version);
}

simulated function CloseWindow()
{
     local WindowConsole C;

     if (TheWindow == None)
         return;

     C = WindowConsole(PlayerPawn(Owner).Player.Console);

     C.bQuickKeyEnable = False;
     C.CloseUWindow();

     if (TheWindow!=None)
     {
          TheWindow.Close();
     }
}

function DestroyWRI()
{
     Destroy();
}

defaultproperties
{
     WindowClass=Class'SmartDMDialogWindow'
     WinWidth=290
     WinHeight=230
}
