//===============================================================================
//   SmartDMScoreboard - The_Cowboy | Modifications by ProAsm - Version 105
//===============================================================================
//
//

class SmartDMScoreBoard extends TournamentScoreBoard config(MutatorPackv1);

#exec texture IMPORT NAME=meter FILE=Textures\meter.pcx GROUP=SmartSB MIPS=OFF
#exec texture IMPORT NAME=bot1 File=Textures\bot1.pcx GROUP=SmartSB
#exec texture IMPORT NAME=altred File=Textures\altred.pcx GROUP=SmartSB
#exec texture IMPORT NAME=altblu File=Textures\altblu.pcx GROUP=SmartSB
#exec texture IMPORT NAME=dead File=Textures\dead1.pcx GROUP=SmartSB
#exec texture IMPORT NAME=redback File=Textures\redback.pcx GROUP=SmartSB MIPS=OFF
#exec texture IMPORT NAME=bluback File=Textures\bluback.pcx GROUP=SmartSB MIPS=OFF
#exec texture IMPORT NAME=dmback File=Textures\dmback.pcx GROUP=SmartSB MIPS=OFF

//User's settings
var() config bool bShowTransparent;
var() config bool bShowAltScoreboard;
var() config bool bStatsDrawFaces;
var() config bool bShowInlineDMPlayers;
var() config bool bShowAltTeamHeading;
var() config bool bShowTickFrameRate;
var() config bool bShowSBCrossHair;

var localized string OrdersString, InString;

var ScoreBoard NormalScoreBoard;
var SmartDMGameReplicationInfo SDMGame;
var SmartDMPlayerReplicationInfo OwnerStats;

var int TryCount, ol, pl;
var PlayerPawn PlayerOwner;

var string DeathsText, FragsText, SepText, MoreText, HeaderText ,CreditText;
var int LastSortTime, MaxMeterWidth;
var byte ColorChangeSpeed, RowColState;
var color GrayColor, DarkGray, YellowColor, RedColor, GoldColor, BlueColor, TealColor,
          RedHeaderColor, BlueHeaderColor, StatsColor, FooterColor, HeaderColor, TinyInfoColor, PurpleColor;
var float StatsTextWidth, StatHeight, MeterHeight, NameHeight, ColumnHeight, StatBlockHeight;
var float LeftStartX, RightStartX ,ColumnWidth, StatWidth, StatsHorSpacing, ShadingSpacingX,
          HeaderShadingSpacingY, ColumnShadingSpacingY;
var float StartY, StatLineHeight, StatBlockSpacing, StatIndent ,MidSpacingPercent;
var TournamentGameReplicationInfo pTGRI;
var PlayerReplicationInfo pPRI;
var Font StatFont, CapFont, FooterFont, GameEndedFont, PlayerNameFont, DeathsFont, TinyFont;
var Font PtsFont22, PtsFont20, PtsFont18, PtsFont16, PtsFont14, PtsFont12;

var int MaxFrags, MaxDeaths ,MaxSpreeEnd ,MaxSuicide ,MaxSurv ,MaxPL ,MaxNetSpeed;
var int TotPlayers;
var bool bStarted;
var bool bEndHandled;
var bool bLMS;
var bool bWide;
var bool bFrame;
var int  iFrame;

var float StartTime;
var float ExtraTime;
var float LastFrameTime;
var float LastSecondStartTime;
var int   FrameCount;
var int   LastSecondFrameCount;
var float MinFPS;
var float MaxFPS;
var float LastSecFPS;

struct FlagData
{
	var string Prefix;
	var texture Tex;
};

var FlagData FD[32]; // there can be max 32 so max 32 different flags
var int saveindex;   // new loaded flags will be saved in FD[index]

function int GetFlagIndex(string Prefix)
{
	local int i;
	for (i=0; i<32; i++)
		if (FD[i].Prefix == Prefix)
		   return i;
	FD[saveindex].Prefix=Prefix;
	FD[saveindex].Tex=texture(DynamicLoadObject(SDMGame.CountryFlagsPackage$"."$Prefix, class'Texture'));
	i=saveindex;
	saveindex = (saveindex+1) % 256;
	return i;
}

function PostBeginPlay()
{
    super.PostBeginPlay();

    SaveConfig();
    PlayerOwner = PlayerPawn( Owner );
    pTGRI = TournamentGameReplicationInfo( PlayerOwner.GameReplicationInfo );
    pPRI = PlayerOwner.PlayerReplicationInfo;
    LastSortTime = -100;

    // Preload
    PtsFont22 = Font( DynamicLoadObject( "LadderFonts.UTLadder22", class'Font' ) );
    PtsFont20 = Font( DynamicLoadObject( "LadderFonts.UTLadder20", class'Font' ) );
    PtsFont18 = Font( DynamicLoadObject( "LadderFonts.UTLadder18", class'Font' ) );
    PtsFont16 = Font( DynamicLoadObject( "LadderFonts.UTLadder16", class'Font' ) );
    PtsFont14 = Font( DynamicLoadObject( "LadderFonts.UTLadder14", class'Font' ) );
    PtsFont12 = Font( DynamicLoadObject( "LadderFonts.UTLadder12", class'Font' ) );

    SpawnNormalScoreBoard();
    if ( NormalScoreBoard == none )
       SetTimer( 1.0 , True );
    else
    {
       bStarted = True;
       SetTimer( 3.0, true);
    }
    Enable('Tick');
}

// Try to spawn a local instance of the original scoreboard class if it doesn't exist already.
function SpawnNormalScoreBoard()
{
    if ( SDMGame == none )
    {
       foreach AllActors( class'SmartDMGameReplicationInfo', SDMGame )
       break;
    }

    if ( SDMGame == none )
       return;

    if ( SDMGame.bCTFGame && !SDMGame.bShowCTFScoreboard )
    {
       NormalScoreBoard = Spawn( SDMGame.NormalScoreBoardClass, PlayerOwner );
       return;
    }

    if ( SDMGame.bAssault && !SDMGame.bAllowAssaultTDM )
    {
       NormalScoreBoard = Spawn( SDMGame.NormalScoreBoardClass, PlayerOwner );
       return;
    }

    if ( SDMGame.bDomination && !SDMGame.bAllowDominationTDM )
    {
       NormalScoreBoard = Spawn( SDMGame.NormalScoreBoardClass, PlayerOwner );
       return;
    }

    //Scoreboard settings
    if ( !SDMGame.bNewFragSystem )
       FragsText = "Score";
    if ( InStr ( pTGRI.GameClass, "LastManStanding" ) != -1 )
    {
       FragsText = "Lives";
       DeathsText = "Kills";
       bLMS = true;
    }

    OwnerStats = SDMGame.GetStats( PlayerOwner );

   if ( SDMGame.NormalScoreBoardClass == none )
    {
       Log( "Unable to identify original ScoreBoard type. Retrying in 1 second." , 'SmartDM' );
       return;
    }

    if ( SDMGame.NormalScoreBoardClass == self.Class )
    {
       NormalScoreBoard = Spawn( class'TournamentScoreBoard', PlayerOwner );
       Log( "Cannot use itself. Using the default DM ScoreBoard instead." , 'SmartDM' );
       return;
    }

    if ( SDMGame.NormalScoreBoardClass != none)
    {
       NormalScoreBoard = Spawn( SDMGame.NormalScoreBoardClass, PlayerOwner );
       Log( "Determined and spawned original scoreboard as" @ NormalScoreBoard, 'SmartDM' );
    }
}

// In the case of the 'normal scoreboard' not being replicated properly, try every second
// to see if it has.
function Timer()
{
    if ( !bStarted )
    {
       if ( NormalScoreBoard == None )
       {
          TryCount++;
          SpawnNormalScoreBoard();
       }

       if ( NormalScoreBoard != None )
       {
          bStarted = True;
          SetTimer( 3.0, True );
       }
       else
       if ( TryCount > 3 )
       {
          Log( "Given up. Using the default ScoreBoard instead." , 'SmartDM' );

          if ( NormalScoreBoard == None )
          {
             NormalScoreBoard = Spawn( class'TournamentScoreBoard', PlayerOwner );
             Log( "Spawned as" @ NormalScoreBoard, 'SmartDM' );
          }
          bStarted = True;
          SetTimer( 3.0, True );
       }
    }
    CheckFrameRate();
}

event Tick( float Delta )
{
    Super.Tick(Delta);

    if (bShowTickFrameRate)
    {
       if (bFrame)
           FrameRateCalc();
    }
}

function CheckFrameRate()
{
    local string sTemp;
    local float f;
	local LevelInfo Entry;

    if (bShowTickFrameRate)
    {
  	   Entry = PlayerOwner.Player.Console.Viewport.Actor.GetEntryLevel();
       if (!bFrame)
       {
          bFrame = true;
		  StartTime = Entry.TimeSeconds;
		  ExtraTime =  0;
		  LastFrameTime = StartTime;
		  LastSecondStartTime = StartTime;
		  FrameCount = 0;
		  LastSecondFrameCount = 0;
		  MinFPS = 0;
		  MaxFPS = 0;
		  LastSecFPS = 0;
       }
       iFrame = int(LastSecFPS);
    }
}

function FrameRateCalc()
{
	local float Delta;
	local LevelInfo Entry;

	Entry = PlayerOwner.Player.Console.Viewport.Actor.GetEntryLevel();

	Delta = Entry.TimeSeconds - LastFrameTime;

	// If delta time is more than a half of a second, ignore frame entirely (precaching, loading etc)
	if( Delta > 0.5 )
	{
		ExtraTime += Delta;
		LastSecondStartTime = Entry.TimeSeconds;
		LastSecondFrameCount = 0;
		LastFrameTime = Entry.TimeSeconds;
		return;
	}

	FrameCount++;
	LastSecondFrameCount++;

	if( Entry.TimeSeconds - LastSecondStartTime > 1)
	{
		LastSecFPS = LastSecondFrameCount / (Entry.TimeSeconds - LastSecondStartTime);
		if( MinFPS == 0 || LastSecFPS < MinFPS )
			MinFPS = LastSecFPS;
		if( LastSecFPS > MaxFPS )
			MaxFPS = LastSecFPS;
		LastSecondFrameCount = 0;
		LastSecondStartTime = Entry.TimeSeconds;
	}

	LastFrameTime = Entry.TimeSeconds;
}

function ShowScores( Canvas C )
{
    if ( SDMGame == none || OwnerStats == none )//|| C.ClipX < 770) // 640x480 is OUT!)
    {
       if ( NormalScoreBoard != none )
          NormalScoreBoard.ShowScores( C );
       else
          PlayerOwner.bShowScores = False;
       return;
    }

    if ( OwnerStats.bEndStats && !bEndHandled )
    {
       bEndHandled = true;
       SetTimer(10, true);
    }

    if ( SDMGame.bCTFGame && !SDMGame.bShowCTFScoreboard)
       OwnerStats.bViewingStats = false;

    if ( SDMGame.bAssault && !SDMGame.bAllowAssaultTDM)
       OwnerStats.bViewingStats = false;

    if ( SDMGame.bDomination && !SDMGame.bAllowDominationTDM)
       OwnerStats.bViewingStats = false;

    if ( OwnerStats.bViewingStats )
    {
       ShowSmartDMScoreBoard( C );
    }
    else
    {
       if ( NormalScoreBoard == none )
          ShowSmartDMScoreBoard( C );
       else
          NormalScoreBoard.ShowScores( C );
    }

    if ( OwnerStats.IndicatorVisibility > 0 )
       ShowIndicator( C );
}

function ShowIndicator( Canvas C )
{
    local float BlockLen, LineHeight;

    C.DrawColor.R = OwnerStats.IndicatorVisibility;
    C.DrawColor.G = OwnerStats.IndicatorVisibility;
    C.DrawColor.B = OwnerStats.IndicatorVisibility;
    C.Style = ERenderStyle.STY_Translucent;
    C.Font = C.SmallFont;
    C.StrLen( "Scoreboards:", BlockLen, LineHeight );
    C.SetPos( C.ClipX - BlockLen - 16, 16 );
    C.DrawText( "Scoreboards:" );
    C.SetPos( C.ClipX - BlockLen, 16 + LineHeight );
    C.DrawText( "Default" );
    C.SetPos( C.ClipX - BlockLen, 16 + 2 * LineHeight );
    C.DrawText( "SmartDM" );

    if ( OwnerStats.bViewingStats )
       C.SetPos( C.ClipX - BlockLen - 16, 16 + 2 * LineHeight );
    else
       C.SetPos( C.ClipX - BlockLen - 16, 16 + LineHeight );

    C.DrawIcon( texture'UWindow.MenuTick', 1 );
    C.Style = ERenderStyle.STY_Normal;

    if ( Level.TimeSeconds - OwnerStats.IndicatorStartShow > 2 )
       OwnerStats.IndicatorVisibility = 0;
}

function ShowSmartDMScoreBoard( Canvas C )
{
    if ( SDMGame.bTeamGame )
    {
       if (SDMGame.bCTFGame)
          SmartCTFShowScores(C);
       else
          SmartTDMShowScores(C);
    }
    else
       SmartDMShowScores(C);
}

function SmartCTFShowScores( Canvas C )
{
    local int ID, i, j, k, l, Secs, Mins;
    local float F, Eff;
    local int X, Y ,LeftY ,RightY ,TempSc, Z, LFlagX, RFlagX, TFlagY;
    local float Nil, DummyX, DummyY, SizeX, SizeY, Buffer, Size;
    local byte LabelDrawn[2], Rendered[2];
    local Color PlayerColor, TempColor, TeamColor;
    local string TempStr;
    local SmartDMPlayerReplicationInfo PlayerStats, PlayerStats2;
    local int FlagShift; /* shifting elements to fit a flag */

    if( Level.TimeSeconds - LastSortTime > 0.5 )
    {
        SortScores(32);
        RecountNumbers();
        InitStatBoardConstPos( C );
        LastSortTime = Level.TimeSeconds;
    }

	if ( bShowSBCrossHair && PlayerOwner.Health > 0 && !pPRI.bWaitingPlayer)
       PlayerOwner.MyHUD.DrawCrossHair(C, 0,0 );

    j = 32;
    Y = int( StartY ) + j-2;
    LeftY  = Y + j;
    RightY = Y + j;

    //Draw Totals background
    Z = ColumnWidth + ( ShadingSpacingX * 2 );  // ColumnWidth = 466 | ShadingSpacingX = 11

    LFlagX = LeftStartX + ShadingSpacingX  * 2.5;
    RFlagX = RightStartX + ShadingSpacingX * 2.5;
    TFlagY = Y - HeaderShadingSpacingY - 2;

    j = 1;
    if (C.ClipX < 1100)
       j = 0;
    C.Style = ERenderStyle.STY_Normal;
    if (bShowTransparent)
       C.Style = ERenderStyle.STY_Translucent;

    C.bNoSmooth = False;

    // Draw Red heading background
    C.Style = ERenderStyle.STY_Modulated;
    if (bShowTransparent)
       C.Style = ERenderStyle.STY_Translucent;
    C.SetPos( LeftStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    if (!bShowTransparent)
    {
       C.DrawColor = GrayColor;
       C.DrawRect( Texture'redback', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
    }
    C.DrawColor = RedHeaderColor;
    C.SetPos( LeftStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    C.Style = ERenderStyle.STY_Translucent;
    C.DrawRect( Texture'tranglow', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
    C.DrawColor = RedColor;
    // Draw Red heading icon
    C.SetPos( LeftStartX, Y - HeaderShadingSpacingY+2);
    C.DrawIcon( texture'I_TeamR', 0.42 );

    // Draw Blue heading background
    C.Style = ERenderStyle.STY_Modulated;
    if (bShowTransparent)
       C.Style = ERenderStyle.STY_Translucent;
    C.SetPos( RightStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    if (!bShowTransparent)
    {
       C.DrawColor = GrayColor;
       C.DrawRect( Texture'bluback', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
    }
    C.DrawColor = BlueHeaderColor;
    C.Style = ERenderStyle.STY_Translucent;
    C.SetPos( RightStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    C.DrawRect( Texture'tranglowb', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
    C.DrawColor = WhiteColor;
    // Draw Blue heading icon
    C.SetPos( RightStartX-1, Y - HeaderShadingSpacingY);
    C.DrawIcon( texture'I_TeamB', 0.45 );

    //Draw total scores heading
    C.Style = ERenderStyle.STY_Normal;
    C.Font = PlayerNameFont;
    C.DrawColor = RedColor;
    if (SDMGame.RedTeamHeading != "")
       TempStr = SDMGame.RedTeamHeading;
    else
       TempStr = "Red Team";
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( ((LeftStartX + Z) / 2) - (buffer / 2) + 10, Y - HeaderShadingSpacingY + 4);
    C.DrawText( TempStr );

    C.Font = MyFonts.GetHugeFont(C.ClipX);
    TempStr = string(int(pTGRI.Teams[0].Score));
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( LeftStartX + ColumnWidth - Size - Buffer - StatsHorSpacing, Y - HeaderShadingSpacingY );
    C.DrawText( TempStr );

    C.Font = PlayerNameFont;
    C.DrawColor = BlueColor;
    if (SDMGame.BlueTeamHeading != "")
       TempStr = SDMGame.BlueTeamHeading;
    else
       TempStr = "Blue Team";
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( (RightStartX + (Z / 2)) - (buffer / 2), Y - HeaderShadingSpacingY + 4);
    C.DrawText( TempStr );

    C.Font = MyFonts.GetHugeFont(C.ClipX);
    TempStr = string(int(pTGRI.Teams[1].Score));
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( RightStartX + ColumnWidth - Size - Buffer - StatsHorSpacing, Y - HeaderShadingSpacingY );
    C.DrawText( TempStr );

    // FOR EACH PLAYER DRAW INFO
    for ( i=0; i<32; i++ )
    {
        if ( Ordered[i] == none )
           break;
        PlayerStats = SDMGame.GetStatsByPRI( Ordered[i] );
        if ( PlayerStats == None )
           continue;

        // Get the ID of the ith player
        ID = Ordered[i].PlayerID;

        // set the pos depending on Rank
        if ( Ordered[I].Team == 0 ) //red team
        {
           TeamColor = RedColor;
           X = LeftStartX;
           Y = LeftY;
        }
        else
        {
           TeamColor = BlueColor;
           X = RightStartX;
           Y = RightY;
        }

        C.DrawColor = TeamColor;
        C.Font = FooterFont;
        C.StrLen( "Test", Nil, DummyY );
        C.Font = PlayerNameFont;
        C.StrLen( "TEST", SizeX, SizeY );

        // DRAW THE INDIVIDUAL SCORES with the cool Flag icons (masked because of black borders)
        Z = SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing;
        j = 6;
        if (bShowAltScoreboard)
            j = 3;
        if ( (LabelDrawn[Ordered[i].Team] != 2)&& Y > C.ClipY - (DummyY * j + Z) )
        {
           C.DrawColor = WhiteColor;
           C.Font = FooterFont;
           C.StrLen( MoreText , Size, DummyY );
           LabelDrawn[Ordered[i].Team] = 2;
 	       if ( Ordered[i].Team == 0 )
 	       {
	          C.SetPos( X - ShadingSpacingX, Y);
	          C.DrawText( "[" @ pTGRI.Teams[0].Size - Rendered[0] @ MoreText @ "]" );
	       }
	       else
	       {
	          C.SetPos( X + ColumnWidth - Size - Len("[ 5 More... ]"), Y);
              C.DrawText( "[" @ pTGRI.Teams[1].Size - Rendered[1] @ MoreText @ "]" );
           }
        }
        else
        if ( LabelDrawn[Ordered[i].Team] != 2 )
        {
           // Draw dark background shading
           C.bNoSmooth = False;
           C.DrawColor = GrayColor;
           C.Style = ERenderStyle.STY_Modulated;
           C.SetPos( X - ShadingSpacingX, Y - HeaderShadingSpacingY );
           j = 22;
           if (bShowAltScoreboard)
               j = -9;
           if (!bShowTransparent)
           {
              if ( Ordered[I].Team == 0 ) //red team
                 C.DrawRect( texture'redback', ColumnWidth + ( ShadingSpacingX * 2 ) , SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing + j);
              else
                 C.DrawRect( texture'bluback', ColumnWidth + ( ShadingSpacingX * 2 ) , SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing + j);
           }
           // Draw the face or team icon
           C.bNoSmooth = True;
           C.DrawColor = WhiteColor;
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           if ( Ordered[i].Team == 0 ) //red team
           {
              if ( bStatsDrawFaces && Ordered[i].TalkTexture != none )
              {
                 C.DrawIcon( Ordered[i].TalkTexture, 0.4 );
              }
              else
              {
                 C.DrawColor = TeamColor;
                 C.DrawIcon( texture'I_TeamR', 0.4 );
              }
           }
           else
           {
              if ( bStatsDrawFaces && Ordered[i].TalkTexture != none )
              {
                 C.DrawIcon( Ordered[i].TalkTexture, 0.4 );
              }
              else
              {
                 C.DrawColor = TeamColor;
                 C.DrawIcon( texture'I_TeamB', 0.4 );
              }
           }
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           C.DrawColor = DarkGray;
           if ( bStatsDrawFaces )
              C.DrawIcon( texture'IconSelection', 0.8 );

           Y -= 4;

           //Draw the Deaths/Frags text
           C.bNoSmooth = True;
           C.DrawColor = TeamColor;
           C.Font = DeathsFont;
           C.StrLen( DeathsText, Buffer, Nil );
           C.SetPos( X + ColumnWidth - SizeX - Buffer - 3, Y );
           C.DrawText( FragsText $ SepText $ DeathsText);

           // Draw the player name
           C.Font = PlayerNameFont;
           C.StrLen( "TEST", Nil, DummyY );
           C.SetPos( X + StatIndent, Y - ( ( DummyY - SizeY ) / 2 ) );
           if ( Ordered[i].bAdmin )
              C.DrawColor = WhiteColor;
           else
           if( Ordered[i].PlayerID == pPRI.PlayerID )
              C.DrawColor = YellowColor;
           else
           {
               C.DrawColor = BlueColor;
               if ( Ordered[I].Team == 0 ) //red team
                  C.DrawColor = RedColor;
           }
           TempColor = C.DrawColor;
           C.DrawText( Ordered[i].PlayerName );

           C.bNoSmooth = True;

           Y += SizeY + HeaderShadingSpacingY + ColumnShadingSpacingY + 2;

           if ( Ordered[I].HasFlag != None )
           {
              j = 10;
              k = -4;
              F = 0.9;
              if ( bShowAltScoreboard )
              {
                 j = 0;
                 k = 5;
                 F = 0.6;
              }
              if ( Ordered[i].Team == 0 )       // red team
                 C.SetPos( LeftStartX+k, Y+j);
              else
                 C.SetPos( RightStartX+k, Y+j);
              C.DrawColor = WhiteColor;
              if ( Ordered[I].Team == 0 ) //red team
              {
                 C.DrawIcon( texture'BlueFlag', F );
                 // Also draw red team has the flag
                 C.SetPos( LFlagX, TFlagY);
                 C.DrawIcon( texture'BlueFlag', 1.1 );
              }
              else
              {
                 C.DrawIcon( texture'RedFlag', F );
                 // Also draw blue team has the flag
                 C.SetPos( RFlagX, TFlagY );
                 C.DrawIcon( texture'RedFlag', 1.1 );
              }
           }

           C.Font = FooterFont;
           C.StrLen( "Test", Nil, DummyY );

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           C.DrawColor = TinyInfoColor;
           C.Font = StatFont;
           C.StrLen( "TEST", Buffer, DummyY );

           // Draw the country flag or the word BOT
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X+6, Y-12);
           if ( C.ClipX < 770 )
              C.SetPos( X+6, Y-8);
           C.bNoSmooth = False;
           C.DrawColor = WhiteColor;
           if (PlayerStats.CountryPrefix != "")
           {
              C.DrawIcon(FD[GetFlagIndex(PlayerStats.CountryPrefix)].Tex, 1.0);
           }
           else
           {
              if ( Ordered[i].bIsABot )
              {
                 C.DrawIcon(texture'bot1', 0.45);
                 if ( PlayerStats.bDead && bShowAltScoreboard )
                    goto pass1;
                 if ( C.ClipX < 770 )
                    goto pass1;
                 if (Ordered[I].HasFlag != None && bShowAltScoreboard )
                    goto pass1;
                 C.Font = TinyFont;
                 C.SetPos( X+5, Y+4);
                 C.DrawColor = TeamColor;
                 C.DrawText( "BOT" );
              }
              else
                 C.DrawIcon(texture'tlsteel', 0.25);
           }
pass1:
           C.DrawColor = WhiteColor;

           if ( PlayerStats.bDead && C.ClipX > 770 ) // player is dead
           {
              j = 18;
              k = 4;
              F = 1.1;
              if ( bShowAltScoreboard )
              {
                 j = 3;
                 k = 6;
                 F = 0.9;
              }
              if ( Ordered[i].Team == 0 )       // red team
                 C.SetPos( LeftStartX+k, Y+j);
              else
                 C.SetPos( RightStartX+k, Y+j);
              C.DrawIcon(texture'dead', F);
           }

           C.bNoSmooth = True;
           C.Font = PlayerNameFont;
           C.DrawColor = TempColor;

           // Draw Death/Frags/Score
           Z = Y - 24;
           if ( !SDMGame.bNewFragSystem )
           {
              C.StrLen( int(Ordered[i].Score), Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( int(Ordered[i].Score) );
           }
           else
           {
              C.StrLen( PlayerStats.Frags, Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( PlayerStats.Frags );
           }

           C.Font = DeathsFont;
           C.StrLen( SepText $ int(Ordered[i].Deaths), Size, SizeY );
           C.SetPos( X + ColumnWidth - Size, Z + 3);
           C.DrawText( SepText $ int(Ordered[i].Deaths));

           Y -= 20;
           Z = X + StatIndent;
           j = 1;
           if (C.ClipX > 1000)
              j = 2;
           if (C.ClipX > 1200)
              j = 4;

           // Draw the Time on   - StatIndent=37.625 | StatsHorSpacing=5.625 | SizeY=21 | DummyY=25
           k = 0;
           if (C.ClipX < 770)
              k = 1;
           C.Font = StatFont;
           C.SetPos( Z, Y+k );
           C.DrawColor = WhiteColor;
           Mins = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) / 60;
           F    = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) - (Mins * 60);
           Secs = int(F / 60 * 10);
           if (Secs < 0)
               Secs = 0;
           TempStr = "";
           TempStr = TempStr$Mins$".";
           TempStr = TempStr$Secs;
           C.DrawText( "Time:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( TempStr );

           k = 12;
           if (C.ClipX < 770)
              k = 10;

           // Draw player ping
           C.SetPos( Z, Y+k );
           C.DrawText( "Ping:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].Ping) );

           k = 24;
           if (C.ClipX < 770)
              k = 19;

           // Draw player loss
           C.SetPos( Z, Y+k );
           C.DrawText( "Plos:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].PacketLoss)$"%" );

           Z += StatIndent;
           k = 0;
           if (C.ClipX < 1000)
              k = 1;

           // Draw Efficiency
           if (C.ClipX > 770)
           {
              if ( PlayerStats.Frags + Ordered[i].Deaths == 0 )
                 Eff = 0;
              else
                 Eff = ( PlayerStats.Frags / ( PlayerStats.Frags + Ordered[i].Deaths ) ) * 100;
              DrawStatType1( C, Z, Y, 1, 1, "Eff:", Eff, 100 ,true);
              DrawStatType1( C, Z, Y+k, 2, 1, "Surv:", int(SelElem(PlayerStats.PlayerStatsString,22)), MaxSurv );
              DrawStatType1( C, Z, Y, 3, 1, "Nett:", PlayerStats.NetSpeed, MaxNetSpeed );
           }

           if (C.ClipX < 770)
              Z -= 25;
           DrawStatType2( C, Z, Y, 1, 2,  "FlgTaken:", int(SelElem(PlayerStats.PlayerStatsString,34)), 0 );
           DrawStatType2( C, Z, Y+k, 2, 2,"Dropped:", int(SelElem(PlayerStats.PlayerStatsString,35)), 0 );
           DrawStatType2( C, Z, Y, 3, 2,  "Captures:", int(SelElem(PlayerStats.PlayerStatsString,36)), 0 );

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           if (!bShowAltScoreboard)
           {
              Y += 36;

              // Draw killing stuff
              k = 2;
              if (C.ClipX < 1000)
                 k = 0;
              if (C.ClipX < 770)
                 k = - 6;
              DrawKillPick( C, Z, Y+k, "Kiltype:", PlayerStats.PlayerStatsString, 23, True);
              Y += 10;
              // Draw pickup stuff
              DrawKillPick( C, Z, Y+k, "Pickups:", PlayerStats.PlayerStatsString, 15, False);

		      // Draw location, Order
		      if (Ordered[i].Team == pPRI.Team && C.ClipX > 770)
		      {
			     C.DrawColor = GreenColor;
			     if ( Ordered[i].PlayerLocation != None )
			   	    TempStr = Ordered[i].PlayerLocation.LocationName;
			     else
			     if ( Ordered[i].PlayerZone != None )
			        TempStr = Ordered[i].PlayerZone.ZoneName;
			     else
			   	    TempStr = "";

			     j = 9;
			     k = C.ClipX / 36;
			     l = 8;
			     if (C.ClipX > 1000)
			     {
			         j = 12;
			         l = 3;
                 }

			     if ( TempStr != "" )
			     {
			   	    TempStr = InString@TempStr;
			        if (Len(TempStr) > k)
			           TempStr = Left(TempStr, k);
			   	    C.SetPos( Z + StatWidth - (StatsHorSpacing*l), Y + j);
			   	    C.DrawText(TempStr, False);              // draw location
			     }
			     TempStr = pTGRI.GetOrderString(Ordered[i]);
			     if (TempStr != "")
			     {
			   	    TempStr = OrdersString@TempStr;
			        if (Len(TempStr) > k)
			           TempStr = Left(TempStr, k);
			   	    C.SetPos( z - StatIndent, Y + j);
			   	    C.DrawText(TempStr, False);              // draw orders
			     }
		      }
           }
           j = 10;
           if (bShowAltScoreboard)
              j = 24;
           if (C.ClipY > 1000)
              Y += StatBlockHeight + StatBlockSpacing + j+2;
           else
              Y += StatBlockHeight + StatBlockSpacing + j;
        }

        // Alter the LeftY or RightY and do next player
        if ( Ordered[I].Team == 0 ) //red team
        {
           LeftY = Y;
           Rendered[0]++;
        }
        else
        {
           RightY = Y;
           Rendered[1]++;
        }
    } //End of PRI for loop

    DrawHeader( C );
    DrawFooters( C );
}

function SmartTDMShowScores( Canvas C )
{
    local int ID, i, j, k, l, Secs, Mins;
    local float F, Eff;
    local int X, Y ,LeftY ,RightY ,TempSc, Z, LFlagX, RFlagX, TFlagY;
    local float Nil, DummyX, DummyY, SizeX, SizeY, Buffer, Size;
    local byte LabelDrawn[2], Rendered[2];
    local Color PlayerColor, TempColor, TeamColor;
    local string TempStr;
    local SmartDMPlayerReplicationInfo PlayerStats, PlayerStats2;
    local int FlagShift; /* shifting elements to fit a flag */

    if( Level.TimeSeconds - LastSortTime > 0.5 )
    {
        SortScores(32);
        RecountNumbers();
        InitStatBoardConstPos( C );
        LastSortTime = Level.TimeSeconds;
    }

	if ( bShowSBCrossHair && PlayerOwner.Health > 0 && !pPRI.bWaitingPlayer)
       PlayerOwner.MyHUD.DrawCrossHair(C, 0,0 );

    j = 32;
    Y = int( StartY ) + j-2;
    LeftY  = Y + j;
    RightY = Y + j;

    //Draw Totals background
    Z = ColumnWidth + ( ShadingSpacingX * 2 );  // ColumnWidth = 466 | ShadingSpacingX = 11

    j = 1;
    if (C.ClipX < 1100)
       j = 0;

    C.bNoSmooth = False;

    // Draw Red heading background
    C.Style = ERenderStyle.STY_Modulated;
    if (bShowTransparent)
       C.Style = ERenderStyle.STY_Translucent;
    C.SetPos( LeftStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    if (bShowAltTeamHeading)
    {
       C.Style = ERenderStyle.STY_Normal;
       if (bShowTransparent)
          C.Style = ERenderStyle.STY_Translucent;
       C.DrawColor = RedColor;
       C.DrawRect( Texture'altred', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       C.SetPos( LeftStartX-1, Y - HeaderShadingSpacingY);
       C.DrawIcon( texture's_corpse', 1.0); //
    }
    else
    {
       if (!bShowTransparent)
       {
          C.DrawColor = GrayColor;
          C.DrawRect( Texture'dmback', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       }
       C.DrawColor = RedHeaderColor;
       C.SetPos( LeftStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
       C.Style = ERenderStyle.STY_Translucent;
       C.DrawRect( Texture'tranglow', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       C.DrawColor = RedColor;
       // Draw Red heading icon
       C.SetPos( LeftStartX-1, Y - HeaderShadingSpacingY);
       C.DrawIcon( texture's_corpse', 1.0); //
    }

    // Draw Blue heading background
    C.Style = ERenderStyle.STY_Modulated;
    if (bShowTransparent)
       C.Style = ERenderStyle.STY_Translucent;
    C.SetPos( RightStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
    if (bShowAltTeamHeading)
    {
       C.Style = ERenderStyle.STY_Normal;
       if (bShowTransparent)
          C.Style = ERenderStyle.STY_Translucent;
       C.DrawColor = BlueColor;
       C.DrawRect( Texture'altblu', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       C.SetPos( RightStartX-1, Y - HeaderShadingSpacingY);
       C.DrawIcon( texture's_corpse', 1.0);
    }
    else
    {
       if (!bShowTransparent)
       {
          C.DrawColor = GrayColor;
          C.DrawRect( Texture'dmback', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       }
       C.DrawColor = BlueHeaderColor;
       C.Style = ERenderStyle.STY_Translucent;
       C.SetPos( RightStartX - ShadingSpacingX - j, Y - HeaderShadingSpacingY - 1);
       C.DrawRect( Texture'tranglowb', Z+1 , SizeY + NameHeight+HeaderShadingSpacingY+1);
       C.DrawColor = BlueColor;
       // Draw Blue heading icon
       C.SetPos( RightStartX-1, Y - HeaderShadingSpacingY);
       C.DrawIcon( texture's_corpse', 1.0);
    }

    //Draw total scores heading
    C.Style = ERenderStyle.STY_Normal;
    C.Font = PlayerNameFont;
    C.DrawColor = RedColor;
    if (bShowAltTeamHeading)
       C.DrawColor = WhiteColor;
    if (SDMGame.RedTeamHeading != "")
       TempStr = SDMGame.RedTeamHeading;
    else
       TempStr = "Red Team";
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( ((LeftStartX + Z) / 2) - (buffer / 2) + 10, Y - HeaderShadingSpacingY + 4);
    C.DrawText( TempStr );

    C.Font = MyFonts.GetHugeFont(C.ClipX);
    TempStr = string(int(pTGRI.Teams[0].Score));
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( LeftStartX + ColumnWidth - Size - Buffer - StatsHorSpacing, Y - HeaderShadingSpacingY );
    C.DrawText( TempStr );

    C.Font = PlayerNameFont;
    C.DrawColor = BlueColor;
    if (bShowAltTeamHeading)
       C.DrawColor = WhiteColor;
    if (SDMGame.BlueTeamHeading != "")
       TempStr = SDMGame.BlueTeamHeading;
    else
       TempStr = "Blue Team";
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( (RightStartX + (Z / 2)) - (buffer / 2), Y - HeaderShadingSpacingY + 4);
    C.DrawText( TempStr );

    C.Font = MyFonts.GetHugeFont(C.ClipX);
    TempStr = string(int(pTGRI.Teams[1].Score));
    C.StrLen( TempStr, Buffer, SizeY );
    C.SetPos( RightStartX + ColumnWidth - Size - Buffer - StatsHorSpacing, Y - HeaderShadingSpacingY );
    C.DrawText( TempStr );

    // FOR EACH PLAYER DRAW INFO
    for ( i=0; i<32; i++ )
    {
        if ( Ordered[i] == none )
           break;
        PlayerStats = SDMGame.GetStatsByPRI( Ordered[i] );
        if ( PlayerStats == None )
           continue;

        // Get the ID of the ith player
        ID = Ordered[i].PlayerID;

        // set the pos depending on Rank
        if ( Ordered[I].Team == 0 ) //red team
        {
           TeamColor = RedColor;
           X = LeftStartX;
           Y = LeftY;
        }
        else
        {
           TeamColor = BlueColor;
           X = RightStartX;
           Y = RightY;
        }

        C.DrawColor = TeamColor;
        C.Font = FooterFont;
        C.StrLen( "Test", Nil, DummyY );
        C.Font = PlayerNameFont;
        C.StrLen( "TEST", SizeX, SizeY );

        // DRAW THE INDIVIDUAL SCORES with the cool Flag icons (masked because of black borders)
        Z = SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing;
        j = 6;
        if (bShowAltScoreboard)
            j = 3;
        if ( (LabelDrawn[Ordered[i].Team] != 2)&& Y > C.ClipY - (DummyY * j + Z) )
        {
           C.DrawColor = WhiteColor;
           C.Font = FooterFont;
           C.StrLen( MoreText , Size, DummyY );
           LabelDrawn[Ordered[i].Team] = 2;
 	       if ( Ordered[i].Team == 0 )
 	       {
	          C.SetPos( X - ShadingSpacingX, Y);
	          C.DrawText( "[" @ pTGRI.Teams[0].Size - Rendered[0] @ MoreText @ "]" );
	       }
	       else
	       {
	          C.SetPos( X + ColumnWidth - Size - Len("[ 5 More... ]"), Y);
              C.DrawText( "[" @ pTGRI.Teams[1].Size - Rendered[1] @ MoreText @ "]" );
           }
        }
        else
        if ( LabelDrawn[Ordered[i].Team] != 2 )
        {
           // Draw dark background shading
           C.bNoSmooth = False;
           C.DrawColor = GrayColor;
           C.Style = ERenderStyle.STY_Modulated;
           C.SetPos( X - ShadingSpacingX, Y - HeaderShadingSpacingY );
           j = 22;
           if (bShowAltScoreboard)
               j = -9;
           if (!bShowTransparent)
               C.DrawRect( texture'dmback', ColumnWidth + ( ShadingSpacingX * 2 ) , SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing + j);

           // Draw the face or team icon
           C.bNoSmooth = True;
           C.DrawColor = WhiteColor;
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           if ( Ordered[i].Team == 0 ) //red team
           {
              if ( bStatsDrawFaces && Ordered[i].TalkTexture != none )
              {
                 C.DrawIcon( Ordered[i].TalkTexture, 0.4 );
              }
              else
              {
                 C.DrawColor = TeamColor;
                 C.DrawIcon( texture'I_TeamR', 0.4 );
              }
           }
           else
           {
              if ( bStatsDrawFaces && Ordered[i].TalkTexture != none )
              {
                 C.DrawIcon( Ordered[i].TalkTexture, 0.4 );
              }
              else
              {
                 C.DrawColor = TeamColor;
                 C.DrawIcon( texture'I_TeamB', 0.4 );
              }
           }
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           C.DrawColor = DarkGray;
           if ( bStatsDrawFaces )
              C.DrawIcon( texture'IconSelection', 0.8 );

           Y -= 4;

           //Draw the Deaths/Frags text
           C.bNoSmooth = True;
           C.DrawColor = TeamColor;
           C.Font = DeathsFont;
           C.StrLen( DeathsText, Buffer, Nil );
           C.SetPos( X + ColumnWidth - SizeX - Buffer - 3, Y );
           C.DrawText( FragsText $ SepText $ DeathsText);

           // Draw the player name
           C.Font = PlayerNameFont;
           C.StrLen( "TEST", Nil, DummyY );
           C.SetPos( X + StatIndent, Y - ( ( DummyY - SizeY ) / 2 ) );
           if ( Ordered[i].bAdmin )
              C.DrawColor = WhiteColor;
           else
           if( Ordered[i].PlayerID == pPRI.PlayerID )
              C.DrawColor = YellowColor;
           else
           {
               C.DrawColor = BlueColor;
               if ( Ordered[I].Team == 0 ) //red team
                  C.DrawColor = RedColor;
           }
           TempColor = C.DrawColor;
           C.DrawText( Ordered[i].PlayerName );

           C.bNoSmooth = True;

           Y += SizeY + HeaderShadingSpacingY + ColumnShadingSpacingY + 2;

           C.Font = FooterFont;
           C.StrLen( "Test", Nil, DummyY );

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           C.DrawColor = TinyInfoColor;
           C.Font = StatFont;
           C.StrLen( "TEST", Buffer, DummyY );

           // Draw the country flag or the word BOT
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X+6, Y-12);
           if ( C.ClipX < 770 )
              C.SetPos( X+6, Y-8);
           C.bNoSmooth = False;
           C.DrawColor = WhiteColor;
           if (PlayerStats.CountryPrefix != "")
           {
              C.DrawIcon(FD[GetFlagIndex(PlayerStats.CountryPrefix)].Tex, 1.0);
           }
           else
           {
              if ( Ordered[i].bIsABot )
              {
                 C.DrawIcon(texture'bot1', 0.45);
                 if ( PlayerStats.bDead && bShowAltScoreboard )
                    goto pass1;
                 if ( C.ClipX < 770 )
                    goto pass1;
                 C.Font = TinyFont;
                 C.SetPos( X+5, Y+4);
                 C.DrawColor = TeamColor;
                 C.DrawText( "BOT" );
              }
              else
                 C.DrawIcon(texture'tlsteel', 0.25);
           }
pass1:
           C.DrawColor = WhiteColor;

           if ( PlayerStats.bDead && C.ClipX > 770 ) // player is dead
           {
              j = 18;
              k = 4;
              F = 1.1;
              if ( bShowAltScoreboard )
              {
                 j = 3;
                 k = 6;
                 F = 0.9;
              }
              if ( Ordered[i].Team == 0 )       // red team
                 C.SetPos( LeftStartX+k, Y+j);
              else
                 C.SetPos( RightStartX+k, Y+j);
              C.DrawIcon(texture'dead', F);
           }

           C.bNoSmooth = True;
           C.Font = PlayerNameFont;
           C.DrawColor = TempColor;

           // Draw Death/Frags/Score
           Z = Y - 24;
           if ( !SDMGame.bNewFragSystem )
           {
              C.StrLen( int(Ordered[i].Score), Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( int(Ordered[i].Score) );
           }
           else
           {
              C.StrLen( PlayerStats.Frags, Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( PlayerStats.Frags );
           }

           C.Font = DeathsFont;
           C.StrLen( SepText $ int(Ordered[i].Deaths), Size, SizeY );
           C.SetPos( X + ColumnWidth - Size, Z + 3);
           C.DrawText( SepText $ int(Ordered[i].Deaths));

           Y -= 20;
           Z = X + StatIndent;
           j = 1;
           if (C.ClipX > 1000)
              j = 2;
           if (C.ClipX > 1200)
              j = 4;

           // Draw the Time on   - StatIndent=37.625 | StatsHorSpacing=5.625 | SizeY=21 | DummyY=25
           k = 0;
           if (C.ClipX < 770)
              k = 1;
           C.Font = StatFont;
           C.SetPos( Z, Y+k );
           C.DrawColor = WhiteColor;
           Mins = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) / 60;
           F    = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) - (Mins * 60);
           Secs = int(F / 60 * 10);
           if (Secs < 0)
               Secs = 0;
           TempStr = "";
           TempStr = TempStr$Mins$".";
           TempStr = TempStr$Secs;
           C.DrawText( "Time:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( TempStr );

           k = 12;
           if (C.ClipX < 770)
              k = 10;

           // Draw player ping
           C.SetPos( Z, Y+k );
           C.DrawText( "Ping:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].Ping) );

           k = 24;
           if (C.ClipX < 770)
              k = 19;

           // Draw player loss
           C.SetPos( Z, Y+k );
           C.DrawText( "Plos:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].PacketLoss)$"%" );

           Z += StatIndent;
           k = 0;
           if (C.ClipX < 1000)
              k = 1;

           // Draw Efficiency
           if (C.ClipX > 770)
           {
              if ( PlayerStats.Frags + Ordered[i].Deaths == 0 )
                 Eff = 0;
              else
                 Eff = ( PlayerStats.Frags / ( PlayerStats.Frags + Ordered[i].Deaths ) ) * 100;
              DrawStatType1( C, Z, Y, 1, 1, "Eff:", Eff, 100 ,true);
              DrawStatType1( C, Z, Y+k, 2, 1, "Surv:", int(SelElem(PlayerStats.PlayerStatsString,22)), MaxSurv );
              DrawStatType1( C, Z, Y, 3, 1, "Nett:", PlayerStats.NetSpeed, MaxNetSpeed );
           }

           if (C.ClipX < 770)
              Z -= 20;
           DrawStatType2( C, Z, Y, 1, 2, "Suicides:",int(SelElem(PlayerStats.PlayerStatsString,14)), 0 );
           DrawStatType2( C, Z, Y+k, 2, 2, "Hedshot:", PlayerStats.HeadShots, 0 ,false);
           DrawSpree( C, Z, Y, 3, 2, "Killspree:", PlayerStats.PlayerStatsString);

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           if (!bShowAltScoreboard)
           {
              Y += 36;

              // Draw killing stuff
              k = 2;
              if (C.ClipX < 1000)
                 k = 0;
              if (C.ClipX < 770)
                 k = - 6;
              DrawKillPick( C, Z, Y+k, "KILTYPE:", PlayerStats.PlayerStatsString, 23, True);
              Y += 10;
              // Draw pickup stuff
              DrawKillPick( C, Z, Y+k, "PICKUPS:", PlayerStats.PlayerStatsString, 15, False);

		      // Draw location, Order
		      if (Ordered[i].Team == pPRI.Team && C.ClipX > 770)
		      {
			     C.DrawColor = GreenColor;
			     if ( Ordered[i].PlayerLocation != None )
			   	    TempStr = Ordered[i].PlayerLocation.LocationName;
			     else
			     if ( Ordered[i].PlayerZone != None )
			        TempStr = Ordered[i].PlayerZone.ZoneName;
			     else
			   	    TempStr = "";

			     j = 9;
			     k = C.ClipX / 36;
			     l = 8;
			     if (C.ClipX > 1000)
			     {
			         j = 12;
			         l = 3;
                 }

			     if ( TempStr != "" )
			     {
			   	    TempStr = InString@TempStr;
			        if (Len(TempStr) > k)
			           TempStr = Left(TempStr, k);
			   	    C.SetPos( Z + StatWidth - (StatsHorSpacing*l), Y + j);
			   	    C.DrawText(TempStr, False);              // draw location
			     }
			     TempStr = pTGRI.GetOrderString(Ordered[i]);
			     if (TempStr != "")
			     {
			   	    TempStr = OrdersString@TempStr;
			        if (Len(TempStr) > k)
			           TempStr = Left(TempStr, k);
			   	    C.SetPos( z - StatIndent, Y + j);
			   	    C.DrawText(TempStr, False);              // draw orders
			     }
		      }
           }
           j = 10;
           if (bShowAltScoreboard)
              j = 24;
           if (C.ClipY > 1000)
              Y += StatBlockHeight + StatBlockSpacing + j+2;
           else
              Y += StatBlockHeight + StatBlockSpacing + j;
        }

        // Alter the LeftY or RightY and do next player
        if ( Ordered[I].Team == 0 ) //red team
        {
           LeftY = Y;
           Rendered[0]++;
        }
        else
        {
           RightY = Y;
           Rendered[1]++;
        }
    } //End of PRI for loop

    DrawHeader( C );
    DrawFooters( C );
}

function SmartDMShowScores( Canvas C )
{
    local int ID, i, j, k, Secs, Mins;
    local float F, Eff;
    local int X, Y ,LeftY ,RightY ,TempSc, Z;
    local float Nil, DummyX, DummyY, SizeX, SizeY, Buffer, Size;
    local byte LabelDrawn, Rendered;
    local Color PlayerColor, TempColor;
    local string TempStr;
    local SmartDMPlayerReplicationInfo PlayerStats, PlayerStats2;
    local int FlagShift;
    local bool bLeft;

    if( Level.TimeSeconds - LastSortTime > 0.5 )
    {
        SortScores( 32 );
        RecountNumbers();
        InitStatBoardConstPos( C );
    }

	if ( bShowSBCrossHair && PlayerOwner.Health > 0 && !pPRI.bWaitingPlayer)
       PlayerOwner.MyHUD.DrawCrossHair(C, 0,0 );

    j = 16;
    Y = int( StartY ) + j-2;
    LeftY  = Y + j;
    RightY = Y + j;
    C.Style = ERenderStyle.STY_Normal;

    // FOR EACH PLAYER DRAW INFO
    for ( i=0; i<32; i++ )
    {
        if ( Ordered[i] == none )
           break;
        PlayerStats = SDMGame.GetStatsByPRI( Ordered[i] );
        if ( PlayerStats == None )
           continue;

        // Get the ID of the ith player
        ID = Ordered[i].PlayerID;

        C.DrawColor = RedColor;
        C.Font = FooterFont;
        C.StrLen( "Test", Nil, DummyY );
        C.Font = PlayerNameFont;
        C.StrLen( "TEST", SizeX, SizeY );

        Z = SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing;
        j = 5;
        if (bShowAltScoreboard)
            j = 3;
        if (bShowInlineDMPlayers)
        {
           bLeft = true;
           if (LeftY > C.ClipY - (DummyY * j + Z))
           {
              X = RightStartX;                    // set the pos depending on Rank
              Y = RightY;
              bLeft = false;
              if (LabelDrawn != 2 && Y > C.ClipY - (DummyY * j + Z) )
              {
                 if ( i%2 == 0 ) //Draw only on middle from Left side (check not needed but still :)
                 {
                    C.bCenter = True;
                    C.DrawColor = WhiteColor;
                    C.Font = FooterFont;
                    C.StrLen( "[" @ TotPlayers - Rendered @ MoreText @ "]" , Size, DummyY );
                    C.SetPos( 0, Y - DummyY * 2 );
                    C.DrawText( "[" @ TotPlayers - Rendered @ MoreText @ "]" );
                    C.bCenter = False;
                    LabelDrawn = 2;
                 }
              }
           }
           else
           {
              X = LeftStartX;
              Y = LeftY;
           }
        }
        else
        {
           if ( i%2 == 0 )//left side
           {
              X = LeftStartX;
              Y = LeftY;
           }
           else
           {
              X = RightStartX;
              Y = RightY;
           }

           // DRAW THE INDIVIDUAL SCORES with the cool Flag icons (masked because of black borders)
           if (LabelDrawn != 2 && Y > C.ClipY - (DummyY * j + Z) )
           {
              if ( i%2 == 0 ) //Draw only on middle from Left side (check not needed but still :)
              {
                 C.bCenter = True;
                 C.DrawColor = WhiteColor;
                 C.Font = FooterFont;
                 C.StrLen( "[" @ TotPlayers - Rendered @ MoreText @ "]" , Size, DummyY );
                 C.SetPos( 0, Y - DummyY * 2 );
                 C.DrawText( "[" @ TotPlayers - Rendered @ MoreText @ "]" );
                 C.bCenter = False;
                 LabelDrawn = 2;
               }
           }
        }
        if (LabelDrawn != 2)
        {
           //Draw dark background
           C.bNoSmooth = False;
           C.DrawColor = GrayColor;
           C.StrLen( FragsText, SizeX, SizeY );
           C.Style = ERenderStyle.STY_Modulated;
           C.SetPos( X - ShadingSpacingX, Y - HeaderShadingSpacingY );
           j = 14;
           if (bShowAltScoreboard)
               j = -7;
           if (!bShowTransparent)
              C.DrawRect( texture'dmback', ColumnWidth + ( ShadingSpacingX * 2 ) , SizeY + HeaderShadingSpacingY*2 + ColumnShadingSpacingY + StatBlockHeight + StatBlockSpacing + j);

           // Draw the face or X
           C.bNoSmooth = True;
           C.DrawColor = WhiteColor;
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           if ( bStatsDrawFaces && Ordered[i].TalkTexture != none )
               C.DrawIcon( Ordered[i].TalkTexture, 0.4 );
           else
           {
               C.DrawColor = DarkGray;
               C.DrawIcon( texture'I_TeamN', 0.4 );
           }
           C.SetPos( X, Y - ( ( 32 - SizeY ) / 2 ) );
           C.DrawColor = DarkGray;
           if ( bStatsDrawFaces )
              C.DrawIcon( texture'IconSelection', 0.8 );

           Y -= 4;

           // Display player number
           C.bNoSmooth = True;
           if ( bLMS && Ordered[i].Score < 1 )
              C.DrawColor = GrayColor;
           else
              C.DrawColor = BlueColor;
           C.Font = PlayerNameFont; //GameEndedFont; //PtsFont22;
           if (C.ClipX < 1000)
              C.Font = PtsFont20;
           C.StrLen(  i+1, DummyX, DummyY );
           C.Style = ERenderStyle.STY_Normal;
           C.SetPos( X + StatIndent, Y - ( ( DummyY - SizeY ) / 2 ) );
           C.DrawText( ( i+1 ) );

           //Draw the Deaths/Frags text
           C.Font = DeathsFont;
           C.StrLen( DeathsText, Buffer, Nil );
           C.SetPos( X + ColumnWidth - SizeX - Buffer - 3, Y );
           C.DrawText( FragsText $ SepText $ DeathsText);

           // Draw the player name
           C.Font = PlayerNameFont;
           C.StrLen( "TEST", Nil, DummyY );
           C.SetPos( X + StatIndent + DummyX + 2 * StatsHorSpacing, Y - ( ( DummyY - SizeY ) / 2 ) );
           if ( Ordered[i].bAdmin )
              C.DrawColor = WhiteColor;
           else
           if ( bLMS && Ordered[i].Score < 1 )
           {
              if ( Ordered[i].PlayerID == pPRI.PlayerID )
                 C.DrawColor = DarkGray;
              else
                 C.DrawColor = GrayColor;
           }
           else
           if ( Ordered[i].PlayerID == pPRI.PlayerID )
              C.DrawColor = YellowColor;
           else
              C.DrawColor = BlueColor;
           TempColor = C.DrawColor;
           C.DrawText( Ordered[i].PlayerName );

           C.bNoSmooth = True;

           Y += SizeY + HeaderShadingSpacingY + ColumnShadingSpacingY + 2;

           C.Font = FooterFont;
           C.StrLen( "Test", Nil, DummyY );

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           C.DrawColor = TinyInfoColor;
           C.Font = StatFont;
           C.StrLen( "TEST", Buffer, DummyY );

           // Draw the country flag or the word BOT
           C.Style = ERenderStyle.STY_Translucent;
           C.SetPos( X+6, Y-12);
           if ( C.ClipX < 770 )
              C.SetPos( X+6, Y-8);
           C.bNoSmooth = False;
           C.DrawColor = WhiteColor;
           if (PlayerStats.CountryPrefix != "")
           {
              C.DrawIcon(FD[GetFlagIndex(PlayerStats.CountryPrefix)].Tex, 1.0);
           }
           else
           {
              if ( Ordered[i].bIsABot )
              {
                 C.DrawIcon(texture'bot1', 0.45);
                 if ( PlayerStats.bDead && bShowAltScoreboard )
                    goto pass2;
                 if ( C.ClipX < 770 )
                    goto pass2;
                 C.Font = TinyFont;
                 C.SetPos( X+5, Y+4);
                 C.DrawColor = BlueColor;
                 C.DrawText( "BOT" );
              }
              else
                 C.DrawIcon(texture'tlsteel', 0.25);
           }
pass2:
           C.DrawColor = WhiteColor;

           if ( PlayerStats.bDead && C.ClipX > 770 ) // player is dead
           {
              j = 18;
              k = 4;
              F = 1.1;
              if ( bShowAltScoreboard )
              {
                 j = 3;
                 k = 6;
                 F = 0.9;
              }
              if (bShowInlineDMPlayers)
              {
                 if (bLeft)
                    C.SetPos( LeftStartX+k, Y+j);
                 else
                    C.SetPos( RightStartX+k, Y+j);
              }
              else
              {
                 if ( i%2 == 0)                // left side
                    C.SetPos( LeftStartX+k, Y+j);
                 else
                    C.SetPos( RightStartX+k, Y+j);
              }
              C.DrawIcon(texture'dead', F);
           }

           C.bNoSmooth = True;
           C.DrawColor = TempColor;

           // Draw Death/Frags/Score
           Z = Y - 24;
           C.Font = PlayerNameFont;
           if ( !SDMGame.bNewFragSystem || bLMS )
           {
              C.StrLen( int(Ordered[i].Score), Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( int(Ordered[i].Score) );
           }
           else
           {
              C.StrLen( PlayerStats.Frags, Size, DummyY );
              C.SetPos( X + ColumnWidth - Size - Buffer - 8, Z );
              C.DrawText( PlayerStats.Frags );
           }
           C.Font = DeathsFont;
           if ( !bLMS )
           {
              C.StrLen( SepText $ int(Ordered[i].Deaths), Size, SizeY );
              C.SetPos( X + ColumnWidth - Size, Z + 3);
              C.DrawText( SepText $ int(Ordered[i].Deaths) );
           }
           else
           {
              C.StrLen( SepText $ PlayerStats.Frags , Size, SizeY );
              C.SetPos( X + ColumnWidth - Size, Z + 3);
           }

           Y -= 20;
           Z = X + StatIndent;
           j = 1;
           if (C.ClipX > 1000)
              j = 2;
           if (C.ClipX > 1200)
              j = 4;

           // Draw the Time on   - StatIndent=37.625 | StatsHorSpacing=5.625 | SizeY=21 | DummyY=25
           k = 0;
           if (C.ClipX < 770)
              k = 1;
           C.Font = StatFont;
           C.SetPos( Z, Y+k );
           C.DrawColor = WhiteColor;
           Mins = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) / 60;
           F    = (Level.TimeSeconds + pPRI.StartTime - Ordered[i].StartTime) - (Mins * 60);
           Secs = int(F / 60 * 10);
           if (Secs < 0)
               Secs = 0;
           TempStr = "";
           TempStr = TempStr$Mins$".";
           TempStr = TempStr$Secs;
           C.DrawText( "Time:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( TempStr );

           k = 12;
           if (C.ClipX < 770)
              k = 10;

           // Draw player ping
           C.SetPos( Z, Y+k );
           C.DrawText( "Ping:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].Ping) );

           k = 24;
           if (C.ClipX < 770)
              k = 19;

           // Draw player loss
           C.SetPos( Z, Y+k );
           C.DrawText( "Plos:" );
           C.SetPos( Z + StatIndent + j, Y+k );
           C.DrawText( string(Ordered[i].PacketLoss)$"%" );

           Z += StatIndent;
           k = 0;
           if (C.ClipX < 1000)
              k = 1;

           // Draw Efficiency
           if (C.ClipX > 770)
           {
              if ( PlayerStats.Frags + Ordered[i].Deaths == 0 )
                 Eff = 0;
              else
                 Eff = ( PlayerStats.Frags / ( PlayerStats.Frags + Ordered[i].Deaths ) ) * 100;
              DrawStatType1( C, Z, Y, 1, 1, "Eff:", Eff, 100 ,true);
              DrawStatType1( C, Z, Y+k, 2, 1, "Surv:", int(SelElem(PlayerStats.PlayerStatsString,22)), MaxSurv );
              DrawStatType1( C, Z, Y, 3, 1, "Nett:", PlayerStats.NetSpeed, MaxNetSpeed );
           }

           if (C.ClipX < 770)
              Z -= 20;
           DrawStatType2( C, Z, Y, 1, 2, "Suicides:",int(SelElem(PlayerStats.PlayerStatsString,14)), 0 );
           DrawStatType2( C, Z, Y+k, 2, 2, "Hedshot:", PlayerStats.HeadShots, 0 ,false);
           DrawSpree( C, Z, Y, 3, 2, "Killspree:", PlayerStats.PlayerStatsString);

           C.Style = ERenderStyle.STY_Normal;
           C.bNoSmooth = True;

           if (!bShowAltScoreboard)
           {
              Y += 36;

              // Draw killing stuff
              k = 2;
              if (C.ClipX < 1000)
                 k = 0;
              if (C.ClipX < 770)
                 k = - 6;
              DrawKillPick( C, Z, Y+k, "KILTYPE:", PlayerStats.PlayerStatsString, 23, True);
              Y += 10;
              // Draw pickup stuff
              DrawKillPick( C, Z, Y+k, "PICKUPS:", PlayerStats.PlayerStatsString, 15, False);
           }
           j = 2;
           if (bShowAltScoreboard)
              j = 26;
           Y += StatBlockHeight + StatBlockSpacing + j;
        }

        // Alter the LeftY or RightY and do next player
        if (bShowInlineDMPlayers)
        {
           if ( bLeft )
              LeftY = Y;
           else
              RightY = Y;
        }
        else
        {
           if ( i%2 == 0 )
              LeftY = Y;
           else
              RightY = Y;
        }
        Rendered++;
    } //End of PRI for loop

    DrawHeader( C );
    DrawFooters( C );
}

function InitStatBoardConstPos( Canvas C )
{
    local float Nil, LeftSpacingPercent, RightSpacingPercent;

    CapFont = Font'LEDFont2';
    FooterFont = MyFonts.GetSmallestFont( C.ClipX );
    GameEndedFont = MyFonts.GetHugeFont( C.ClipX );
    PlayerNameFont = MyFonts.GetBigFont( C.ClipX );
    TinyFont = C.SmallFont;
    StatFont = Font(DynamicLoadObject("LadderFonts.UTLadder12", class'Font'));

    if ( PlayerNameFont == PtsFont22 )
       DeathsFont = PtsFont18;
    else
    if ( PlayerNameFont == PtsFont20 )
       DeathsFont = PtsFont16;
    else
    if ( PlayerNameFont == PtsFont18 )
       DeathsFont = PtsFont14;
    else
    if ( PlayerNameFont == PtsFont16 )
       DeathsFont = PtsFont12;
    else
       DeathsFont = font'SmallFont';

    if (C.ClipX < 1000)
    {
       PlayerNameFont = PtsFont18;
       DeathsFont = PtsFont14;
       StatFont = TinyFont;
    }

    C.Font = PlayerNameFont;
    C.StrLen( "Player", Nil, NameHeight );

    StartY = ( 120.0 / 1024.0 ) * C.ClipY;
    ColorChangeSpeed = 100; // Influences how 'fast' the color changes from white to green. Higher = faster.

    LeftSpacingPercent = 0.045;
    MidSpacingPercent = 0.10;
    RightSpacingPercent = 0.045;
    LeftStartX = LeftSpacingPercent * C.ClipX;
    ColumnWidth = ( ( 1 - LeftSpacingPercent - MidSpacingPercent - RightSpacingPercent ) / 2 * C.ClipX );
    RightStartX = LeftStartX + ColumnWidth + ( MidSpacingPercent * C.ClipX );
    ShadingSpacingX = ( 10.0 / 1024.0 ) * C.ClipX;
    HeaderShadingSpacingY = ( 32 - NameHeight ) / 2 + ( ( 4.0 / 1024.0 ) * C.ClipX );
    ColumnShadingSpacingY = ( 10.0 / 1024.0 ) * C.ClipX;

    StatsHorSpacing = ( 5.0 / 1024.0 ) * C.ClipX;
    StatIndent = ( 32 + StatsHorSpacing ); // For face + flag icons

    if (C.ClipX / C.ClipY > 1.5)           // test for wide screen
        bWide = true;

    InitStatBoardDynamicPos( C );
}

function InitStatBoardDynamicPos( Canvas C , optional int Rows , optional int Cols , optional Font NewStatFont , optional float LineSpacing , optional float BlockSpacing )
{
    local font xStatFont;

    if ( Rows == 0 )
       Rows = 3;
    if ( Cols == 0 )
       Cols = 2;
    if ( LineSpacing == 0 )
       LineSpacing = 0.9;
    if ( BlockSpacing == 0 )
       BlockSpacing = 1;

    if ( Rows == 2 && Cols == 3 )
       RowColState = 1;
    else
       RowColState = 0;

    StatWidth = ( ( ColumnWidth - StatIndent ) / Cols ) - ( StatsHorSpacing * ( Cols - 1 ) );

    if ( NewStatFont == none )
       xStatFont = MyFonts.GetSmallestFont( C.ClipX );
    else
       xStatFont = NewStatFont;   // It's just a lot easier to leave this as a dummy :)

    C.Font = xStatFont;
    C.StrLen( "FlagKls: 00", StatsTextWidth, StatHeight );

    MaxMeterWidth = StatWidth - StatsTextWidth - StatsHorSpacing;  // 132 on 864
    if ( C.ClipX > 1100)
       MaxMeterWidth = MaxMeterWidth * 0.5;
    else
       MaxMeterWidth = MaxMeterWidth * 0.4;
    StatLineHeight = StatHeight * LineSpacing; // StatLineHeight=14.4 | StatHeight = 16 | LineSpacing = 0.9
    MeterHeight = Max( 1, StatLineHeight * 0.3 );
    StatBlockSpacing = StatLineHeight * BlockSpacing;
    StatBlockHeight = Rows * StatLineHeight - 32;

    if ( pTGRI.NumPlayers % 2 == 0 )
       ColumnHeight = ( pTGRI.NumPlayers/2 ) * ( NameHeight + StatBlockHeight + StatBlockSpacing ) - StatBlockSpacing;
    else
       ColumnHeight = ( pTGRI.NumPlayers/2 + 1 ) * ( NameHeight + StatBlockHeight + StatBlockSpacing ) - StatBlockSpacing;
}

/*
 * Draw a specific stat
 * X, Y = Upper left corner of stats ( row,col: 1,1)
 */
function DrawStatType1( Canvas C, int X, int Y, int Row, int Col, string Label, int Count, int Total ,optional bool bIsPercent)
{
    local float Size, DummyY;
    local int ColorChange, M, j;

    X += StatIndent + ( ( StatWidth + StatsHorSpacing ) * ( Col - 1 ) );
    Y += ( (StatLineHeight-2) * ( Row - 1 ) );

    C.SetPos( X, Y );
    C.DrawText( Label );

    if (C.ClipX > 1200)
        X += 12;

    C.StrLen( string(Count)$"%", Size, DummyY );

    if (Caps(Left(Label, 3)) == "NET")
    {
       j = 23;
       if (C.ClipX > 1200)
           j = 23;
       if (C.ClipX < 900)
           j = 28;
       C.SetPos( X + StatsTextWidth - j, Y);
       C.DrawText( Count );
       return;
    }

    j = 5;

    if ( bIsPercent )                    // StatsTextWidth=71 | Size=15 | StatsHorSpacing=5.625
    {
       C.SetPos( X + StatsTextWidth - Size - j, Y );
       C.DrawText( string(Count)$"%" ); //text
    }
    else
    {
       if (Col == 1)
          C.SetPos( X + StatsTextWidth - Size - j, Y );
       else
          C.SetPos( X + StatsTextWidth, Y );
       if (Count > 999)
           Count = 999;
       C.DrawText( Count ); //text
    }

    if (C.ClipX < 1000)
        return;

    if ( Count > 0 )
    {
       ColorChange = ColorChangeSpeed * loge( Count );
       if ( ColorChange > 255 )
          ColorChange = 255;
       C.DrawColor.R = StatsColor.R - ColorChange;
       C.DrawColor.B = StatsColor.B - ColorChange;
    }                                                        // StatsTextWidth = 71 StatsHorSpacing = 5.625

    M = GetMeterLength( Count, Total );
    C.SetPos( X + StatsTextWidth + StatsHorSpacing, Y + ( ( StatHeight - MeterHeight ) / 2 ) );
    C.DrawRect( texture'meter', M, MeterHeight ); //meter
    C.DrawColor = WhiteColor;
}

function DrawStatType2( Canvas C, int X, int Y, int Row, int Col, string Label, int Count, int Total ,optional bool bIsPercent)
{
    local float Size, DummyY;
    local int Perc, j;

    X += StatWidth - (StatsHorSpacing*3);
    Y += ( (StatLineHeight-2) * ( Row - 1 ) );

    if (C.ClipX < 1000)
        X -= 20;

    C.DrawColor = StatsColor;
    C.SetPos( X, Y );
    C.DrawText( Label );

    j = 0; // for 1024 ?
    if (C.ClipX < 800 || C.ClipX > 1100)
        j = 12;

    C.SetPos( X + StatsTextWidth + j, Y );
    C.DrawText( Count );
}
function DrawSpree( Canvas C, int X, int Y, int Row, int Col, string Label, string Str  )
{
    local float Size, Sx, DummyY ,DummyY2, F;
    local string Temp ,SpreeType[5];
    local int i, p, j, k, Count, z;

    SpreeType[0] = "K";
    SpreeType[1] = "R";
    SpreeType[2] = "D";
    SpreeType[3] = "U";
    SpreeType[4] = "G";

    X += StatWidth - (StatsHorSpacing*3);
    Y += ( (StatLineHeight-2) * ( Row - 1 ) );

    Temp = "";
    k = 0;
    z = 0;

    if (C.ClipX < 1000)
        X -= 20;

    C.DrawColor = StatsColor;
    C.SetPos( X, Y );
    C.StrLen( "M", Size, DummyY2 );
    C.DrawText( Label );

    i = 29;
    j = 33;
    if ( SDMGame.bAltMultiKillScores )
    {
       i = 2;
       j = 6;
    }

    while( i <= j )
    {
        p = int( SelElem( Str ,i++ ) );
        if ( p > 0 )
        {
           Temp = Temp@SpreeType[k]$p;
           z++;
        }
        Count += p;
        k++;
        if (z == 4 && C.ClipX < 1000)
           break;
    }

    j = 12;
    if (C.ClipX < 1200)
        j = 0;

    C.Font = StatFont;
    C.SetPos( X + StatsTextWidth + j, Y );
    C.DrawText( Count ); //text

    C.Font = TinyFont;
    C.DrawColor = GrayColor;
    //Temp = " K2 R1 D1 U1 G1"; // test

    F = 1.5;
    if (C.ClipX > 1100)
        F = 3;
    if (C.ClipX > 1200)
        F = 5;

    if (C.ClipX < 1000)
        Y += 3;
    C.StrLen( Temp, Sx, DummyY );
    C.SetPos( X + StatsTextWidth + (Size * F), Y + DummyY2 - DummyY - 3 );
    C.DrawText( Temp ); //text
    C.DrawColor = StatsColor;
}

function DrawKillPick( Canvas C, int X, int Y, string Label, string Str, int iNum, bool bKill )
{
    local float CSize, WSize, DummyY;
    local string sTemp ,KillType[7], PickType[7];
    local Color TempColor, iColor[7];
    local int i, j, k;

    TempColor = C.DrawColor;

    if ( bKill && SDMGame.bAltMultiKillScores )
       iNum = 7;

    iColor[0] = YellowColor;
    iColor[1] = TealColor;
    iColor[2] = GreenColor;
    iColor[3] = WhiteColor;
    iColor[4] = BlueColor;
    iColor[5] = PurpleColor;
    iColor[6] = GrayColor;

    KillType[0] = "DUBL:";
    KillType[1] = "TRPL:";
    KillType[2] = "MULT:";
    KillType[3] = "MEGA:";
    KillType[4] = "ULTA:";
    KillType[5] = "MONS:";
    KillType[6] = "";

    PickType[0] = "BELT:";
    PickType[1] = "HEAL:";
    PickType[2] = "ARMR:";
    PickType[3] = "INVS:";
    PickType[4] = "TPAD:";
    PickType[5] = "AMPL:";
    PickType[6] = "BOOT:";

    C.Font = TinyFont;
    C.SetPos( X - StatIndent, Y);
    C.DrawColor = GoldColor;
    C.DrawText( Label );

    X += StatIndent;

    C.StrLen( "MEGA:", WSize, DummyY );
    C.StrLen( "M", CSize, DummyY );

    if (C.ClipX < 1000)
       X -= CSize*2;
    k = 3;
    if (C.ClipX > 1200)
        k = 4;

    for (i=0; i<7; i++)
    {
        if ( i == 4 && C.ClipX < 770)
           break;
        if ( i == 5 && C.ClipX < 1000)
           break;
        if ( i == 6 )
           if (C.ClipX < 1100 || bKill )
              break;
        j = int( SelElem( Str, i+iNum ) );
        if ( bKill )
           DrawItem(C, X, Y, KillType[i], j, iColor[i]);
        else
           DrawItem(C, X, Y, PickType[i], j, iColor[i]);
        X += WSize + CSize * k;
    }

    C.DrawColor = TempColor;
}

function DrawItem(Canvas C, float X, float Y, string sItem, int Count, Color iColr)
{
    C.SetPos( X, Y);
    C.DrawColor = DarkGray;
    if ( Count > 0)
       C.DrawColor = iColr;
    C.DrawText( sItem$Count );
}

function DrawFooters( Canvas C )
{
    local float DummyX, DummyY, Nil, X1, Y1;
    local string TextStr;
    local string TimeStr;
    local int Hours, Minutes, Seconds, i;
    local PlayerReplicationInfo PRI;

    C.bCenter = True;
    C.Font = FooterFont;
    C.DrawColor = FooterColor;
    C.StrLen( "Test", DummyX, DummyY );

	// Draw Spectators
	if (SDMGame.bShowSpecs)
	{
		for ( i=0; i<32; i++ )
		{
			if (PlayerPawn(Owner).GameReplicationInfo.PRIArray[i] != None)
			{
				PRI = PlayerPawn(Owner).GameReplicationInfo.PRIArray[i];
				if (PRI.bIsSpectator && !PRI.bWaitingPlayer && PRI.StartTime > 0)
				{
					if (HeaderText == "")
					   HeaderText = pri.Playername;
					else
					   HeaderText = HeaderText$", "$pri.Playername;
				}
			}
		}
		if (HeaderText == "")
		    HeaderText = "No Spectators.";
		else
		    HeaderText = HeaderText$".";

        C.StrLen( HeaderText, DummyX, Nil );
        C.Style = ERenderStyle.STY_Normal;
        C.SetPos( 0, C.ClipY - DummyY * 4);        // bottom line - 3

        if (SDMGame.bShowSpecs)// && !bShowAltScoreboard)
        {
           C.DrawColor = TealColor;
           C.Font = MyFonts.GetSmallestFont(C.ClipX);
           C.DrawText("Spectators:"@HeaderText);
        }
	}

	HeaderText="";

    // Draw Time Remaining etc
    if ( bTimeDown || ( PlayerOwner.GameReplicationInfo.RemainingTime > 0 ) )
    {
       bTimeDown = True;
       if ( PlayerOwner.GameReplicationInfo.RemainingTime <= 0 )
       {
          TimeStr = RemainingTime $ "00:00";
       }
       else
       {
          Minutes = PlayerOwner.GameReplicationInfo.RemainingTime / 60;
          Seconds = PlayerOwner.GameReplicationInfo.RemainingTime % 60;
          TimeStr = RemainingTime $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
       }
    }
    else
    {
       Seconds = PlayerOwner.GameReplicationInfo.ElapsedTime;
       Minutes = Seconds / 60;
       Hours = Minutes / 60;
       Seconds = Seconds - ( Minutes * 60 );
       Minutes = Minutes - ( Hours * 60 );
       TimeStr = ElapsedTime $ TwoDigitString( Hours ) $ ":" $ TwoDigitString( Minutes ) $ ":" $ TwoDigitString( Seconds );
    }
    C.DrawColor = FooterColor;
    C.SetPos( 0, C.ClipY - DummyY*3 );          // bottom line - 2
    C.DrawText( "Current Time:" @ GetTimeStr() @ "|" @ TimeStr );

	// Draw Server Info or local game info
    C.bCenter = True;
    C.Font = FooterFont;
    C.DrawColor = FooterColor;
    C.StrLen( "Test", DummyX, DummyY );
    C.SetPos( 0, C.ClipY - DummyY*2 );          // bottom line - 1

	if ( (Level.NetMode == NM_Standalone) && Level.Game.IsA('DeathMatchPlus') )
	{
	   // Display local info in bottom center
	   if ( DeathMatchPlus(Level.Game).bRatedGame )
		  C.DrawText(DeathMatchPlus(Level.Game).RatedGameLadderObj.SkillText@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
	   else
	   if ( DeathMatchPlus(Level.Game).bNoviceMode )
		  C.DrawText(class'ChallengeBotInfo'.default.Skills[Level.Game.Difficulty]@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
	   else
		  C.DrawText(class'ChallengeBotInfo'.default.Skills[Level.Game.Difficulty + 4]@PlayerOwner.GameReplicationInfo.GameName@MapTitle@MapTitleQuote$Level.Title$MapTitleQuote, true);
	}
	else
	{
       // Display server info in bottom center
       TextStr = "Playing" @ Level.Title @ "on" @ pTGRI.ServerName;
       C.DrawText( TextStr );
    }

    // Draw Credit stuff + Tick/Frame rates
    TextStr = "";
    if (bShowTickFrameRate)
    {
       TextStr = TextStr $ "[ Tick Rate:" @ SDMGame.TickRate @ "| ";
       TextStr = TextStr $ "Frame Rate:" @ iFrame @ "]";
    }
    CreditText = "[ SmartDM v"$SDMGame.Version$" | F3 Toggles Scoreboards | Mutate SmartDM Menu ]"$TextStr;
    C.StrLen( CreditText, DummyX, DummyY );
    C.Style = ERenderStyle.STY_Normal;
    C.SetPos( 0, C.ClipY - DummyY );            // bottom line
    C.Font = MyFonts.GetSmallestFont(C.ClipX);
    C.DrawColor = GreenColor;
    C.DrawText( CreditText );

    C.bCenter = False;
}

function DrawHeader( Canvas C )
{
    local float DummyX, DummyY , ScoreStart;

    if ( pTGRI.GameEndedComments == "" )
    {
       C.DrawColor = WhiteColor;
	   C.Font = MyFonts.GetHugeFont(C.ClipX);
       C.bCenter = True;
	   ScoreStart = 5.0/1024.0 * C.ClipY;
       C.SetPos(0, ScoreStart);
	   DrawVictoryConditions(C);
	   return;
    }

    C.Font = GameEndedFont;
    C.StrLen( pTGRI.GameEndedComments, DummyX, DummyY );
    C.DrawColor.R = 0;
    C.DrawColor.G = 0;
    C.DrawColor.B = 64;
    C.Style = ERenderStyle.STY_Normal;
    C.SetPos( C.ClipX / 2 - DummyX / 2 + 3, DummyY + 3 );
    C.DrawText( pTGRI.GameEndedComments );
    C.DrawColor = HeaderColor;
    C.SetPos( C.ClipX / 2 - DummyX / 2, DummyY );
    C.DrawText( pTGRI.GameEndedComments );
}

function DrawVictoryConditions(Canvas Canvas)
{
	local TournamentGameReplicationInfo TGRI;
	local string sGame, sColr;
	local Color MyColor;
	local float XL, YL;
	local int i;

	TGRI = TournamentGameReplicationInfo(PlayerPawn(Owner).GameReplicationInfo);
	if ( TGRI == None )
		return;

    MyColor = Canvas.DrawColor;

    sGame = "";
    sColr = "";

    if (SDMGame != None)
    {
        for (i=0; i<30; i++)
        {
            if ( SDMGame.iGameEnab[i] == 1 && Caps(SDMGame.sGameClas[i]) == Caps(SDMGame.ThisClass) )
            {
                sGame = SDMGame.sGameName[i];
                sColr = SDMGame.sGameColr[i];
                break;
            }
        }
    }

    if (sGame == "" && SDMGame != None)
    {
        for (i=0; i<30; i++)
        {   // if bEnabled and smatdm's gameclass is the same as level.gameclass
            if ( SDMGame.iGameEnab[i] == 1 && Caps(SDMGame.sGameClas[i]) == Caps(SDMGame.ThisClass) )
            {
                sGame = SDMGame.sGameName[i];
                sColr = SDMGame.sGameColr[i];
                break;
            }
            else // if bEnabled and smartdm's gameclass is none and smartdm's gamename is same as level.gamename
            if ( SDMGame.iGameEnab[i] == 1 && Caps(SDMGame.sGameClas[i]) == "" && Caps(SDMGame.sGameName[i]) == Caps(TGRI.GameName) )
            {
                sGame = SDMGame.sGameName[i];
                sColr = SDMGame.sGameColr[i];
                break;
            }
        }
    }

    if (sGame != "")
    {
        MyColor = CheckMyColor(Canvas, sColr);
        Canvas.DrawColor = MyColor;
        if (Caps(sGame) != "NONE")
           Canvas.DrawText(sGame);
        else
           Canvas.DrawText(TGRI.GameName);
    }
    else
	    Canvas.DrawText(TGRI.GameName);

	Canvas.StrLen("Test", XL, YL);
	Canvas.SetPos(0, Canvas.CurY - YL - 2);

	if ( SDMGame.bCTFGame && SDMGame.bShowCTFScoreboard)
	{
	   if ( TGRI.GoalTeamScore > 0 )
	   {
		  Canvas.DrawText("Capture Limit:"@TGRI.GoalTeamScore);
		  Canvas.StrLen("Test", XL, YL);
		  Canvas.SetPos(0, Canvas.CurY - YL - 4);
	   }
	   if ( TGRI.TimeLimit > 0 )
		  Canvas.DrawText(TimeLimit@TGRI.TimeLimit$":00");
	}
	else
	if ( TGRI.FragLimit > 0 )
	{
	   if ( bLMS )
       {
          Canvas.DrawText("Lives Limit"@TGRI.FragLimit);
          if ( TGRI.TimeLimit > 0 )
          {
             Canvas.SetPos(0, Canvas.CurY - YL - 4);
	 	     Canvas.DrawText(TimeLimit@TGRI.TimeLimit$":00");
	 	  }
       }
       else
       {
          Canvas.DrawText(FragGoal@TGRI.FragLimit);
          if ( TGRI.TimeLimit > 0 )
          {
             Canvas.SetPos(0, Canvas.CurY - YL - 4);
		     Canvas.DrawText(TimeLimit@TGRI.TimeLimit$":00");
          }
       }
    }
}

function color CheckMyColor(Canvas C, string MyCol)
{
    local string sTemp;
    local int i, j, k;
    local color cCol;

    cCol = C.DrawColor;
    if (MyCol != "" && Len(MyCol) > 11)
    {
 	   sTemp = MyCol;
	   i = InStr(sTemp, "=");
	   j = InStr(sTemp, ",");
	   k = j - (i+1);
	   cCol.R = Int(Mid(sTemp, i+1, k));

	   sTemp = Mid(sTemp, j+1);
	   i = InStr(sTemp, "=");
	   j = InStr(sTemp, ",");
	   k = j - (i+1);
	   cCol.G = Int(Mid(sTemp, i+1, k));

	   sTemp = Mid(sTemp, j+1);
	   i = InStr(sTemp, "=");
	   cCol.B = Int(Mid(sTemp, i+1));
    }
    if (cCol.R == 0 && cCol.G == 0 && cCol.B == 0)
        cCol = C.DrawColor;
    return cCol;
}

/*
 * Returns time and date in a string.
 */
function string GetTimeStr()
{
    local string Mon, Day, Min;

    Min = string( PlayerOwner.Level.Minute );
    if ( int( Min ) < 10 )
       Min = "0" $ Min;

    switch( PlayerOwner.Level.month )
    {
      case  1: Mon = "Jan"; break;
      case  2: Mon = "Feb"; break;
      case  3: Mon = "Mar"; break;
      case  4: Mon = "Apr"; break;
      case  5: Mon = "May"; break;
      case  6: Mon = "Jun"; break;
      case  7: Mon = "Jul"; break;
      case  8: Mon = "Aug"; break;
      case  9: Mon = "Sep"; break;
      case 10: Mon = "Oct"; break;
      case 11: Mon = "Nov"; break;
      case 12: Mon = "Dec"; break;
    }

    switch( PlayerOwner.Level.dayOfWeek )
    {
      case 0: Day = "Sunday";    break;
      case 1: Day = "Monday";    break;
      case 2: Day = "Tuesday";   break;
      case 3: Day = "Wednesday"; break;
      case 4: Day = "Thursday";  break;
      case 5: Day = "Friday";    break;
      case 6: Day = "Saturday";  break;
    }

    return Day @ PlayerOwner.Level.Day @ Mon @ PlayerOwner.Level.Year $ "," @ PlayerOwner.Level.Hour $ ":" $ Min;
}

/*
 * Length of a meter drawing for a given number A out of B total.
 */
function int GetMeterLength( int A, int B )
{
    local int Result;

    if ( B == 0 )
       return 0;
    Result = ( A * MaxMeterWidth ) / B;

    if ( Result > MaxMeterWidth )
       return MaxMeterWidth;
    else
       return Result;
}

/*
 * Sort PlayerReplicationInfo's on score.
 */
function SortScores( int N )
{
    local byte i, j;
    local bool bSorted;
    local SmartDMPlayerReplicationInfo PlayerStats1, PlayerStats2;

    // Copy PRI array except for spectators.
    j = 0;

    for ( i=0; i<N; i++ )
    {
        if ( pTGRI.priArray[i] == none )
           break;
        if ( pTGRI.priArray[i].bIsSpectator && !pTGRI.priArray[i].bWaitingPlayer )
           continue;
        Ordered[j] = pTGRI.priArray[i];
        j++;
    }
    // Clear the remaining entries.
    for ( i=j; i<N; i++ )
    {
        Ordered[i] = None;
    }

    for ( i=0; i<N; i++)
    {
        bSorted = True;
        for ( j=0; j<N-1; j++)
        {
            if ( Ordered[j+1] == none )
               break;
            if ( Ordered[j].Score < Ordered[j+1].Score )
            {
               SwapOrdered( j, j + 1 );
               bSorted = False;
            }
            else
            if ( Ordered[j].Score == Ordered[j+1].Score )
            {
               PlayerStats1 = SDMGame.GetStatsByPRI( Ordered[j] );
               PlayerStats2 = SDMGame.GetStatsByPRI( Ordered[j+1] );
               if ( PlayerStats1 != None && PlayerStats2 != None )
               {
                  if ( PlayerStats1.Frags < PlayerStats2.Frags )
                  {
                     SwapOrdered( j, j + 1 );
                     bSorted = False;
                  }
                  else
                  if( PlayerStats1.Frags == PlayerStats2.Frags )
                  {
                     if ( Ordered[j].Deaths > Ordered[j+1].Deaths )
                     {
                       SwapOrdered( j, j + 1 );
                       bSorted = False;
                     }
                  }
               }
            }
        }
        if ( bSorted )
           break;
    }
    LastSortTime = Level.TimeSeconds;
}
/*
 * Used for sorting.
 */
function SwapOrdered( byte A, byte B )
{
    local PlayerReplicationInfo Temp;
    Temp = Ordered[A];
    Ordered[A] = Ordered[B];
    Ordered[B] = Temp;
}

/*
 * Recalculate the totals for displaying meters on the scoreboards.
 * This way it doesn't get calculated every tick.
 */
function RecountNumbers()
{
    local byte ID, i;
    local SmartDMPlayerReplicationInfo PlayerStats;
    local string temp;

    MaxFrags = 0;
    MaxDeaths = 0;
    MaxSpreeEnd = 0;
    MaxSuicide = 0;
    MaxSurv = 0;
    MaxNetSpeed=0;
    MaxPl=0;
    TotPlayers = 0;

    for ( i = 0; i < 32; i++ )
    {
        if ( Ordered[i] == None )
           break;
        if ( Ordered[i].bIsSpectator && !Ordered[i].bWaitingPlayer )
           continue;

        ID = Ordered[i].PlayerID;

        PlayerStats = SDMGame.GetStatsByPRI( Ordered[i] );
        if ( PlayerStats != None )
        {
           if ( PlayerStats.Frags > MaxFrags )
              MaxFrags = PlayerStats.Frags;
           if ( PlayerStats.NetSpeed > MaxNetSpeed )
              MaxNetSpeed = PlayerStats.NetSpeed;
           temp =  SelElem(PlayerStats.PlayerStatsString ,13);
           if ( int( temp ) > MaxSpreeEnd )
              MaxSpreeEnd = int( temp );
           temp =  SelElem(PlayerStats.PlayerStatsString ,14);
           if ( int( temp ) > MaxSuicide )
              MaxSuicide = int( temp );
           temp = SelElem(PlayerStats.PlayerStatsString,22);
           if ( int( temp ) > MaxSurv )
              MaxSurv = int( temp );
           TotPlayers++;
        }
        if ( Ordered[i].Deaths > MaxDeaths )
           MaxDeaths = Ordered[i].Deaths;
        if ( Ordered[i].PacketLoss > MaxPL )
           MaxPL = Ordered[i].PacketLoss;
    }
}

static final function string SelElem(string Str, int Elem)
{
	local int pos;
	while(Elem-->1)
		Str=Mid(Str, InStr(Str,":")+1);
	pos=InStr(Str, ":");
	if(pos != -1)
    	Str=Left(Str, pos);
    return Str;
}

function SetDefaultDisplayProperties()
{
   local SmartDMMenuInfo SWRI;
   local SmartDMMenuInfo A;

   PlayerOwner = PlayerPawn( Owner );

   // check if window already open
   foreach AllActors(class'SmartDMMenuInfo', A)
   {
      if (PlayerOwner == A.Owner)
      {
         A.CloseWindow();
         A.Destroy();
      }
   }

   SWRI = Spawn(class'SmartDMMenuInfo', PlayerOwner);

   if (SWRI == None)
      Log("Oops! Nothing spawned. Do something!!");
   else
   {
      SWRI.Version = SDMGame.Version;
   }
}

defaultproperties
{
     bStatsDrawFaces=True
     bShowAltTeamHeading=True
     bShowTickFrameRate=True
     bShowSBCrossHair=True
     OrdersString="Orders:"
     InString="Location:"
     DeathsText="Deaths"
     FragsText="Frags"
     SepText=" / "
     MoreText="More..."
     GrayColor=(R=144,G=144,B=144)
     DarkGray=(R=36,G=36,B=36)
     YellowColor=(R=255,G=255)
     RedColor=(R=255)
     GoldColor=(R=192,G=112)
     BlueColor=(G=128,B=255,A=255)
     TealColor=(G=255,B=255)
     RedHeaderColor=(R=112)
     BlueHeaderColor=(G=48,B=96)
     StatsColor=(R=255,G=255,B=255)
     FooterColor=(R=255,G=255,B=255)
     HeaderColor=(R=255,G=255)
     TinyInfoColor=(R=128,G=128,B=128)
     PurpleColor=(R=180,G=64,B=255)
}
